#include "pkt_sx.h"
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
#define PKT_SX_SENSOR_VALID_ASSERT 5
static argv_t argv ;
////////////////////////////////////////////////////////////////////////////
static int pkt_sx_parse_sensor ( unsigned char * buf , int size , xlist * result )
{
  static int int32_right_pulse = 0 , int32_left_pulse = 0 ;
  static unsigned short last_right_pulse = 0 , last_left_pulse = 0 ;
  static int ultra_valid_count = 0 , infrared_valid_count = 0 ;
  static int parse_counter = 0 ;
  
  int ultral_valid , infrared_valid ;
  int inc_right_pulse , inc_left_pulse ;
  unsigned char * p , i ;
  unsigned int timestamp ;
  unsigned short right_pulse , left_pulse ;
  short v , count = 0 , angle ,
  rp[10] , lp[10] ,
  ifrared_r , ifrared_l ,
  ultra_l , ultra_m , ultra_r ,
  power,
  right_speed , left_speed ,
  right_crush_1byte , left_crush_1byte ,
  dangling_1byte ,
  drop_mide , drop_right , drop_left , drop_back_right , drop_back_left ;
  
  parse_counter ++ ;
  if ( parse_counter < 128 )
  {
    //ignore the first 128 pkt
    return 1 ;
  }
  
  p = buf ;
  
  //check type
  IN8(p, v) ;
  if ( v != PKT_SX_TYPE_SENSOR )
  {
    return 0 ;
  }
  
  //check length
  IN8(p, v) ;
  if ( v != PKT_SX_SIZE_SENSOR )
  {
    return 0 ;
  }
  
  //got value
  IN32_BE(p, timestamp) ;
  SKIP(p, 1) ;
  IN16_BE(p, angle) ;
  for ( i = 0 ; i < 10 ; i ++ )
  {
    IN16_BE(p, rp[i]);
  }
  for ( i = 0 ; i < 10 ; i ++ )
  {
    IN16_BE(p, lp[i]);
  }
  IN16_BE(p, ifrared_r) ;
  IN16_BE(p, ifrared_l) ;
  IN16_BE(p, ultra_r) ;
  IN16_BE(p, ultra_m) ;
  IN16_BE(p, ultra_l) ;
  IN16_BE(p, power) ;
  IN16_BE(p, right_pulse) ;
  IN16_BE(p, left_pulse) ;
  IN16_BE(p, right_speed) ;
  IN16_BE(p, left_speed) ;
  IN8(p, right_crush_1byte) ;
  IN8(p, left_crush_1byte) ;
  IN8(p, dangling_1byte) ;
  IN16_BE(p, drop_mide) ;
  IN16_BE(p, drop_right) ;
  IN16_BE(p, drop_left) ;
  IN16_BE(p, drop_back_right) ;
  IN16_BE(p, drop_back_left) ;
  
  assert(p - buf == PKT_SX_SIZE_SENSOR + 2) ;

#if DBG_PRINT_FETCH_BUFFER
  xmessage("\n--- sensor ---\n") ;
#endif
  
  //odometer calc
  if ( left_pulse == 0xffff && right_pulse == 0 )
  {
    left_pulse = right_pulse = 0 ;
  }
  
  //1. calc inc
  inc_left_pulse = left_pulse - last_left_pulse ;
  inc_right_pulse = right_pulse - last_right_pulse ;
  
  //2. set last
  last_left_pulse = left_pulse ;
  last_right_pulse = right_pulse ;
  
  //3. correct inc
  //2 -> 65533 ==> 65533 - 2 = 65531 ==> -4
  //65534 -> 4 ==> 4 - 65534 = -65530 ==> 6
  if ( inc_left_pulse > 60000 )
  {
    inc_left_pulse -= 65536 ;
  }
  if (inc_left_pulse < -60000 )
  {
    inc_left_pulse += 65536 ;
  }
  if ( inc_right_pulse > 60000 )
  {
    inc_right_pulse -= 65536 ;
  }
  if (inc_right_pulse < -60000 )
  {
    inc_right_pulse += 65536 ;
  }
  
  //4. set to int32_pulse
  int32_left_pulse += inc_left_pulse ;
  int32_right_pulse += inc_right_pulse ;
  
  argv.type = ARGV_TYPE_SENSOR_ODOMETER ;
  argv.argv[0] = timestamp ;
  argv.argv[1] = int32_left_pulse ;
  argv.argv[2] = int32_right_pulse ;
  argv.count = 3 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //gycoscope
  if ( angle > 1800 )
  {
    angle = angle - 3600 ;
  }
  angle = -angle ;
  
  argv.type = ARGV_TYPE_SENSOR_GYROSCOPE ;
  argv.argv[0] = angle ;
  argv.count = 1 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //infrared
  ifrared_r = ifrared_r < PKT_SX_INFRARED_ACTIVE_DISTANCE ? 1 : 0 ;
  ifrared_l = ifrared_l < PKT_SX_INFRARED_ACTIVE_DISTANCE ? 1 : 0 ;
  
  infrared_valid = ifrared_r + ifrared_l ;
  if ( infrared_valid )
  {
    infrared_valid_count ++ ;
  }
  else
  {
    infrared_valid_count = 0 ;
  }
  
  if ( infrared_valid_count < PKT_SX_SENSOR_VALID_ASSERT )
  {
    ifrared_l = ifrared_r = 0 ;
  }
  
  argv.type = ARGV_TYPE_SENSOR_INFRARED ;
  argv.argv[0] = 0 ;
  argv.argv[1] = 0 ;
  argv.argv[2] = ifrared_r ;
  argv.argv[3] = 0 ;
  argv.argv[4] = 0 ;
  argv.argv[5] = 0 ;
  argv.argv[6] = 0 ;
  argv.argv[7] = 0 ;
  argv.argv[8] = 0 ;
  argv.argv[9] = 0 ;
  argv.argv[10] = ifrared_l ;
  argv.argv[11] = 0 ;
  argv.count = 12 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //ultra
  if ( ultra_l > PKT_SX_ULTRAL_VALID_DISTANCE_MAX )
  {
    ultra_l = 0 ;
  }
  if ( ultra_r > PKT_SX_ULTRAL_VALID_DISTANCE_MAX )
  {
    ultra_r = 0 ;
  }
  if ( ultra_m > PKT_SX_ULTRAL_VALID_DISTANCE_MAX )
  {
    ultra_m = 0 ;
  }
  if ( !ultra_l || !ultra_r )
  {
    ultra_m = 0 ;
  }
  
  ultral_valid = ultra_l + ultra_r + ultra_m ;
  if ( ultral_valid )
  {
    ultra_valid_count ++ ;
  }
  else
  {
    ultra_valid_count = 0 ;
  }
  if ( ultra_valid_count < PKT_SX_SENSOR_VALID_ASSERT )
  {
    ultra_m = ultra_r = ultra_l = 0 ;
  }
  
  argv.type = ARGV_TYPE_SENSOR_ULTRA ;
  argv.argv[0] = ultra_m ;
  argv.argv[1] = ultra_r ;
  argv.argv[2] = 0 ;
  argv.argv[3] = 0 ;
  argv.argv[4] = 0 ;
  argv.argv[5] = 0 ;
  argv.argv[6] = 0 ;
  argv.argv[7] = 0 ;
  argv.argv[8] = 0 ;
  argv.argv[9] = 0 ;
  argv.argv[10] = 0 ;
  argv.argv[11] = ultra_l ;
  argv.count = 12 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //crush
  argv.type = ARGV_TYPE_SENSOR_CRUSH ;
  argv.argv[0] = left_crush_1byte ; argv.argv[1] = right_crush_1byte ;
  argv.count = 2 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //dangling
  argv.type = ARGV_TYPE_SENSOR_CRUSH ;
  argv.argv[0] = dangling_1byte ;
  argv.count = 1 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  //drop
  argv.type = ARGV_TYPE_SENSOR_ULTRA ;
  argv.argv[0] = drop_left ;
  argv.argv[1] = drop_mide ;
  argv.argv[2] = drop_right ;
  argv.argv[3] = drop_back_right ;
  argv.argv[4] = drop_back_left ;
  argv.count = 5 ;
  
  xlist_cat(result, NULL, XLIST_STRING, xmemdup(&argv, sizeof(argv))) ;
  count += argv.count ;
  
  return count ;
}
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
int pkt_parse_sx ( xchain * data , xlist * result )
{
  static unsigned char buf[PKT_MAX_SIZE + 32] ;
  unsigned char * p , tag , type ;
  int ret , length = 0 , step = 1 , pkt_parse_ret = PKT_PARSE_RESULT_RETRY ;
  
  //min 3 byte
  if ( xchain_size(data) < 3 )
  {
    return PKT_PARSE_RESULT_WAITDATA ;
  }
  
  //match pkt
  ret = pkt_match_tag(data, PKT_SX_START_TAG) ;
  if ( ret <= 0 )
  {
    return PKT_PARSE_RESULT_WAITDATA ;
  }
  
  /*
   pkt
   -------------------------------------------------------------------------
   |0xc5|cmd_id(1byte)|length(1byte)|data 1 ~ data n(xbyte)|chencsum(1byte)|
   -------------------------------------------------------------------------
   */
  
  //check head
  p = buf ;
  ret = 0 ;
  
  xchain_get(data, buf, 3) ;
  IN8(p, tag) ;
  assert(tag == PKT_SX_START_TAG) ;
  
  IN8(p, type) ;
  IN8(p, length) ;
  
  //calc whole pkt length = start tag(1) + cmd_id(1) + lenght(1) + data(x) + chencsum(1)
  length += 4 ;
  if( length < PKT_MIN_SIZE || length > PKT_SX_MAXSIZE )
  {
#if DBG_PRINT_PKT_LOG
    xmessage("parse sx : pkt length = %d , invalid pkt!\n" , length ) ;
#endif
    goto __finish ;
  }
  if ( length > xchain_size(data) )
  {
#if DBG_PRINT_PKT_LOG
    xmessage("parse sx : pkt lenght = %d , but cache left size = %d.\n" , length , xchain_size(data) ) ;
#endif
    
    pkt_parse_ret = PKT_PARSE_RESULT_WAITDATA ;
    step = 0 ;
    
    goto __finish ;
  }
  
  //get whole pkt
  xchain_get(data, buf, length) ;
  
  //check checksum
  if ( pkt_check_sum(buf, length - 1, buf[length - 1]) <= 0 )
  {
#if DBG_PRINT_PKT_LOG
    xmessage("parse sx : checksum failed!\n") ;
#endif
    goto __finish ;
  }
  
  //process cmd od
  switch (type)
  {
    case PKT_SX_TYPE_SENSOR:
      ret = pkt_sx_parse_sensor(buf + 1 , length - 2 , result) ;//skip start tag & checksum
      if ( ret <= 0 )
      {
#if DBG_PRINT_PKT_LOG
        xmessage("parse sx : laser pkt parse failed!\n") ;
#endif
        break ;
      }
      
      step = length ;
      pkt_parse_ret = PKT_PARSE_RESULT_OK ;
      break ;
      
    case PKT_SX_TYPE_STATE:
    case PKT_SX_TYPE_RESPOND:
#if DBG_PRINT_PKT_LOG
      xmessage("parse sx : ctrl respond / state.\n") ;
#endif
      step = length ;
      pkt_parse_ret = PKT_PARSE_RESULT_OK ;
      break ;
      
    default:
#if DBG_PRINT_PKT_LOG
      xmessage("parse sx : pkt type is 0x%x, unsupport!\n" , type) ;
#endif
      break ;
  }
  
  //finish
__finish:
  
  xchain_delete(data, step) ;
  return pkt_parse_ret ;
}
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////